@@ -80,6 +80,8 @@ static int spa_v4l2_open(struct impl *this)
 	if (xioctl(port->fd, VIDIOC_QUERYCAP, &port->cap) < 0) {
 		err = errno;
 		spa_log_error(port->log, "QUERYCAP: %m");
+		close(port->fd);
+		port->fd = -1;
 		return -err;
 	}
 
@@ -87,6 +89,8 @@ static int spa_v4l2_open(struct impl *this)
 	    ((port->cap.capabilities & V4L2_CAP_DEVICE_CAPS) &&
 	     (port->cap.device_caps & V4L2_CAP_VIDEO_CAPTURE) == 0)) {
 		spa_log_error(port->log, "v4l2: %s is no video capture device", props->device);
+		close(port->fd);
+		port->fd = -1;
 		return -ENODEV;
 	}
 
@@ -326,8 +330,8 @@ static const struct format_info format_info[] = {
 
 	/* compressed formats */
 	{V4L2_PIX_FMT_MJPEG, FORMAT_ENCODED, VIDEO, MJPG},
-	{V4L2_PIX_FMT_JPEG, FORMAT_ENCODED, IMAGE, JPEG},
-	{V4L2_PIX_FMT_PJPG, FORMAT_UNKNOWN, VIDEO, RAW},
+	{V4L2_PIX_FMT_JPEG, FORMAT_ENCODED, VIDEO, MJPG},
+	{V4L2_PIX_FMT_PJPG, FORMAT_ENCODED, VIDEO, MJPG},
 	{V4L2_PIX_FMT_DV, FORMAT_ENCODED, VIDEO, DV},
 	{V4L2_PIX_FMT_MPEG, FORMAT_ENCODED, VIDEO, MPEGTS},
 	{V4L2_PIX_FMT_H264, FORMAT_ENCODED, VIDEO, H264},
@@ -1194,7 +1198,8 @@ static void v4l2_on_fd_events(struct spa_source *source)
 	if (source->rmask & SPA_IO_ERR) {
 		struct port *port = &this->out_ports[0];
 		spa_log_error(this->log, "v4l2 %p: error %d", this, source->rmask);
-		spa_loop_remove_source(port->data_loop, &port->source);
+		if (port->source.loop)
+			spa_loop_remove_source(port->data_loop, &port->source);
 		return;
 	}
 
@@ -1470,7 +1475,8 @@ static int do_remove_source(struct spa_loop *loop,
 			    void *user_data)
 {
 	struct port *port = user_data;
-	spa_loop_remove_source(port->data_loop, &port->source);
+	if (port->source.loop)
+		spa_loop_remove_source(port->data_loop, &port->source);
 	return 0;
 }
 
